@@ -180,7 +180,7 @@ inform <- function(message = NULL,
 }
 #' @rdname abort
 #' @export
-signal <- function(message,
+signal <- function(message = "",
                    class,
                    ...,
                    .subclass = deprecated()) {
@@ -206,7 +206,7 @@ default_message_file <- function() {
     return(opt)
   }
 
-  if (is_interactive() &&
+  if ((is_interactive() || is_rstudio()) &&
       sink.number("output") == 0 &&
       sink.number("message") == 2) {
     stdout()
@@ -215,6 +215,10 @@ default_message_file <- function() {
   }
 }
 
+is_rstudio <- function() {
+  Sys.getenv("RSTUDIO_SESSION_PID") %in% c(Sys.getpid(), getppid())
+}
+
 deprecate_subclass <- function(subclass, fn, env = caller_env()) {
   msg <- sprintf(
     "The %s argument of %s has been renamed to %s.",
@@ -271,14 +275,19 @@ validate_signal_args <- function(message,
     }
   }
 
-  if (is_null(message)) {
-    if (is_null(class)) {
-      abort("Either `message` or `class` must be supplied.", call = env)
+  if (is_null(message) && is_null(class)) {
+    abort("Either `message` or `class` must be supplied.", call = env)
+  }
+
+  message <- message %||% ""
+  if (is_function(message)) {
+    if (!"..." %in% names(formals(message))) {
+      abort("`cnd_header()` methods must take `...`.", call = env)
     }
-    message <- ""
+  } else {
+    check_character(message, call = env)
   }
 
-  check_character(message, call = env)
   if (!is_null(class)) {
     check_character(class, call = env)
   }
@@ -318,12 +327,7 @@ needs_signal <- function(frequency,
       format_arg(".frequency")
     ))
   }
-  if (!is_string(id)) {
-    abort(sprintf(
-      "%s must be a string.",
-      format_arg(".frequency")
-    ))
-  }
+  check_string(id, arg = ".frequency")
 
   sentinel <- env[[id]]
   if (is_null(sentinel)) {
@@ -351,15 +355,41 @@ peek_verbosity <- function(opt, call = caller_env()) {
   )
 }
 
+#' @rdname abort
+#' @param id The identifying string of the condition that was supplied
+#'   as `.frequency_id` to `warn()` or `inform()`.
+#' @export
+reset_warning_verbosity <- function(id) {
+  reset_verbosity(id, "warning")
+}
+#' @rdname abort
+#' @export
+reset_message_verbosity <- function(id) {
+  reset_verbosity(id, "message")
+}
+reset_verbosity <- function(id, type = c("message", "warning")) {
+  check_string(id)
+  type <- arg_match(type)
+
+  env <- switch(
+    type,
+    message = message_freq_env,
+    warning = warning_freq_env
+  )
+  env[[id]] <- NULL
+
+  invisible(NULL)
+}
+
 message_freq <- function(message, frequency, type) {
   if (is_string(frequency, "always")) {
     return(chr())
   }
 
   if (is_string(frequency, "regularly")) {
-    info <- silver("This %s is displayed once every 8 hours.")
+    info <- col_silver("This %s is displayed once every 8 hours.")
   } else {
-    info <- silver("This %s is displayed once per session.")
+    info <- col_silver("This %s is displayed once per session.")
   }
   sprintf(info, type)
 }
