@@ -973,10 +973,6 @@ nchar_infix <- function(x) {
 #' - Use [as_label()] when you need to transform any kind of object to
 #'   a string to _represent_ that object with a short description.
 #'
-#' Expect `as_name()` to gain
-#' [name-repairing](https://principles.tidyverse.org/names-attribute.html#minimal-unique-universal)
-#' features in the future.
-#'
 #' @param x A string or symbol, possibly wrapped in a [quosure][quosure].
 #'   If a string, the attributes are removed, if any.
 #' @return A character vector of length 1.
@@ -1034,8 +1030,10 @@ call_deparse_highlight <- function(call, arg) {
     args_list <- sprintf("%s = %s", arg, as_label(call[[arg]]))
     args_list <- format_arg_unquoted(args_list)
   } else {
-    args_list <- args_deparse(node_cdr(call))
-    args_list <- substring(args_list, 2, nchar(args_list) - 1)
+    args_list <- call
+    args_list[[1]] <- quote(F)
+    args_list <- as_label(args_list)
+    args_list <- substring(args_list, 3, nchar(args_list) - 1)
   }
 
   head <- call[[1]]
