@@ -453,7 +453,8 @@ quo_name <- function(quo) {
 }
 
 quo_squash_impl <- function(x, parent = NULL, warn = FALSE) {
-  switch_expr(x,
+  switch_expr(
+    x,
     language = {
       if (is_quosure(x)) {
         if (!is_false(warn)) {
@@ -466,11 +467,11 @@ quo_squash_impl <- function(x, parent = NULL, warn = FALSE) {
           warn <- FALSE
         }
 
-        while (is_quosure(x)) {
+        while (is_quosure(maybe_missing(x))) {
           x <- quo_get_expr(x)
         }
         if (!is_null(parent)) {
-          node_poke_car(parent, x)
+          node_poke_car(parent, maybe_missing(x))
         }
         quo_squash_impl(x, parent, warn = warn)
       } else {
@@ -485,14 +486,14 @@ quo_squash_impl <- function(x, parent = NULL, warn = FALSE) {
     }
   )
 
-  x
+  maybe_missing(x)
 }
 
 
 #' @export
 print.quosure <- function(x, ...) {
   cat_line(.trailing = FALSE,
-    bold("<quosure>"),
+    style_bold("<quosure>"),
     "expr: "
   )
   quo_print(x)
@@ -570,7 +571,7 @@ quo_deparse <- function(x, lines = new_quo_deparser()) {
 
 new_quo_deparser <- function(width = peek_option("width"),
                              max_elements = 5L,
-                             crayon = has_crayon()) {
+                             crayon = has_ansi()) {
   lines <- new_lines(
     width = width,
     max_elements = max_elements,
@@ -651,7 +652,7 @@ quo_env_print <- function(env) {
   nm <- env_label(env)
 
   if (!is_reference(env, global_env()) && !is_reference(env, empty_env())) {
-    nm <- blue(nm)
+    nm <- col_blue(nm)
   }
 
   cat_line(nm)
