@@ -1,12 +1,14 @@
 static r_obj* auto_name_call;
-static r_obj* dots_homonyms_arg;
 static r_obj* dots_homonyms_values;
-static r_obj* dots_ignore_empty_arg;
 static r_obj* dots_ignore_empty_values;
 static r_obj* empty_spliced_arg;
 static r_obj* glue_unquote_fn;
 static r_obj* quosures_attrib;
 static r_obj* splice_box_attrib;
+static r_obj* abort_dots_homonyms_ns_sym;
+
+static struct r_lazy dots_homonyms_arg;
+static struct r_lazy dots_ignore_empty_arg;
 
 r_obj* rlang_ns_get(const char* name);
 
