@@ -2,8 +2,8 @@
 #include "internal.h"
 
 
-static sexp* quo_mask_flag_sym = NULL;
-static sexp* data_mask_flag_sym = NULL;
+static r_obj* quo_mask_flag_sym = NULL;
+static r_obj* data_mask_flag_sym = NULL;
 
 enum rlang_mask_type {
   RLANG_MASK_DATA,     // Full data mask
@@ -12,24 +12,24 @@ enum rlang_mask_type {
 };
 
 struct rlang_mask_info {
-  sexp* mask;
+  r_obj* mask;
   enum rlang_mask_type type;
 };
 
-static struct rlang_mask_info mask_info(sexp* mask) {
-  if (r_typeof(mask) != r_type_environment) {
+static struct rlang_mask_info mask_info(r_obj* mask) {
+  if (r_typeof(mask) != R_TYPE_environment) {
     return (struct rlang_mask_info) { r_null, RLANG_MASK_NONE };
   }
 
-  sexp* flag;
+  r_obj* flag;
 
   flag = r_env_find_anywhere(mask, data_mask_flag_sym);
-  if (flag != r_unbound_sym) {
+  if (flag != r_syms.unbound) {
     return (struct rlang_mask_info) { flag, RLANG_MASK_DATA };
   }
 
   flag = r_env_find_anywhere(mask, quo_mask_flag_sym);
-  if (flag != r_unbound_sym) {
+  if (flag != r_syms.unbound) {
     return (struct rlang_mask_info) { flag, RLANG_MASK_QUOSURE };
   }
 
@@ -37,32 +37,32 @@ static struct rlang_mask_info mask_info(sexp* mask) {
 }
 
 
-static sexp* data_pronoun_class = NULL;
-static sexp* ctxt_pronoun_class = NULL;
-static sexp* data_mask_env_sym = NULL;
+static r_obj* data_pronoun_class = NULL;
+static r_obj* ctxt_pronoun_class = NULL;
+static r_obj* data_mask_env_sym = NULL;
 
-static sexp* rlang_new_data_pronoun(sexp* mask) {
-  sexp* pronoun = KEEP(r_new_vector(r_type_list, 1));
+static r_obj* rlang_new_data_pronoun(r_obj* mask) {
+  r_obj* pronoun = KEEP(r_alloc_list(1));
 
   r_list_poke(pronoun, 0, mask);
-  r_poke_attribute(pronoun, r_class_sym, data_pronoun_class);
+  r_attrib_poke(pronoun, r_syms.class_, data_pronoun_class);
 
   FREE(1);
   return pronoun;
 }
-static sexp* rlang_new_ctxt_pronoun(sexp* top) {
-  sexp* pronoun = KEEP(r_new_environment(r_env_parent(top), 0));
+static r_obj* rlang_new_ctxt_pronoun(r_obj* top) {
+  r_obj* pronoun = KEEP(r_alloc_environment(0, r_env_parent(top)));
 
-  r_poke_attribute(pronoun, r_class_sym, ctxt_pronoun_class);
+  r_attrib_poke(pronoun, r_syms.class_, ctxt_pronoun_class);
 
   FREE(1);
   return pronoun;
 }
 
-void poke_ctxt_env(sexp* mask, sexp* env) {
-  sexp* ctxt_pronoun = r_env_find(mask, data_mask_env_sym);
+void poke_ctxt_env(r_obj* mask, r_obj* env) {
+  r_obj* ctxt_pronoun = r_env_find(mask, data_mask_env_sym);
 
-  if (ctxt_pronoun == r_unbound_sym) {
+  if (ctxt_pronoun == r_syms.unbound) {
     r_abort("Internal error: Can't find context pronoun in data mask");
   }
 
@@ -70,62 +70,62 @@ void poke_ctxt_env(sexp* mask, sexp* env) {
 }
 
 
-static sexp* empty_names_chr;
+static r_obj* empty_names_chr;
 
-static void check_unique_names(sexp* x) {
+static void check_unique_names(r_obj* x) {
   // Allow empty lists
   if (!r_length(x)) {
     return ;
   }
 
-  sexp* names = r_names(x);
+  r_obj* names = r_names(x);
   if (names == r_null) {
     r_abort("`data` must be uniquely named but does not have names");
   }
-  if (r_vec_find_first_duplicate(names, empty_names_chr, NULL)) {
+  if (vec_find_first_duplicate(names, empty_names_chr, NULL)) {
     r_abort("`data` must be uniquely named but has duplicate columns");
   }
 }
-sexp* rlang_as_data_pronoun(sexp* x) {
+r_obj* ffi_as_data_pronoun(r_obj* x) {
   int n_kept = 0;
 
   switch (r_typeof(x)) {
-  case r_type_logical:
-  case r_type_integer:
-  case r_type_double:
-  case r_type_complex:
-  case r_type_character:
-  case r_type_raw:
-    x = KEEP_N(r_vec_coerce(x, r_type_list), n_kept);
+  case R_TYPE_logical:
+  case R_TYPE_integer:
+  case R_TYPE_double:
+  case R_TYPE_complex:
+  case R_TYPE_character:
+  case R_TYPE_raw:
+    x = KEEP_N(r_vec_coerce(x, R_TYPE_list), &n_kept);
     // fallthrough
-  case r_type_list:
+  case R_TYPE_list:
     check_unique_names(x);
-    x = KEEP_N(r_list_as_environment(x, r_empty_env), n_kept);
+    x = KEEP_N(r_list_as_environment(x, r_envs.empty), &n_kept);
     break;
-  case r_type_environment:
+  case R_TYPE_environment:
     break;
   default:
     r_abort("`data` must be an uniquely named vector, list, data frame or environment");
   }
 
-  sexp* pronoun = rlang_new_data_pronoun(x);
+  r_obj* pronoun = rlang_new_data_pronoun(x);
 
   FREE(n_kept);
   return pronoun;
 }
 
 
-static sexp* data_mask_top_env_sym = NULL;
+static r_obj* data_mask_top_env_sym = NULL;
 
-static void check_data_mask_input(sexp* env, const char* arg) {
-  if (r_typeof(env) != r_type_environment) {
+static void check_data_mask_input(r_obj* env, const char* arg) {
+  if (r_typeof(env) != R_TYPE_environment) {
     r_abort("Can't create data mask because `%s` must be an environment", arg);
   }
 }
-static void check_data_mask_top(sexp* bottom, sexp* top) {
-  sexp* cur = bottom;
+static void check_data_mask_top(r_obj* bottom, r_obj* top) {
+  r_obj* cur = bottom;
 
-  while (cur != r_empty_env) {
+  while (cur != r_envs.empty) {
     if (cur == top) {
       return ;
     }
@@ -135,39 +135,39 @@ static void check_data_mask_top(sexp* bottom, sexp* top) {
   r_abort("Can't create data mask because `top` is not a parent of `bottom`");
 }
 
-static sexp* env_sym = NULL;
-static sexp* old_sym = NULL;
-static sexp* mask_sym = NULL;
+static r_obj* env_sym = NULL;
+static r_obj* old_sym = NULL;
+static r_obj* mask_sym = NULL;
 
-static sexp* tilde_fn = NULL;
-static sexp* restore_mask_fn = NULL;
+static r_obj* tilde_fn = NULL;
+static r_obj* restore_mask_fn = NULL;
 
-static void on_exit_restore_lexical_env(sexp* mask, sexp* old, sexp* frame) {
-  sexp* fn = KEEP(r_clone(restore_mask_fn));
+static void on_exit_restore_lexical_env(r_obj* mask, r_obj* old, r_obj* frame) {
+  r_obj* fn = KEEP(r_clone(restore_mask_fn));
 
-  sexp* env = KEEP(r_new_environment(r_base_env, 2));
+  r_obj* env = KEEP(r_alloc_environment(2, r_envs.base));
   r_env_poke(env, mask_sym, mask);
   r_env_poke(env, old_sym, old);
   r_fn_poke_env(fn, env);
 
-  sexp* call = KEEP(r_new_call(fn, r_null));
+  r_obj* call = KEEP(r_new_call(fn, r_null));
   r_on_exit(call, frame);
 
   FREE(3);
 }
 
-sexp* rlang_new_data_mask(sexp* bottom, sexp* top) {
-  sexp* data_mask;
+r_obj* ffi_new_data_mask(r_obj* bottom, r_obj* top) {
+  r_obj* data_mask;
 
   if (bottom == r_null) {
-    bottom = KEEP(r_new_environment(r_empty_env, 0));
+    bottom = KEEP(r_alloc_environment(100, r_envs.empty));
     data_mask = bottom;
   } else {
     check_data_mask_input(bottom, "bottom");
     // Create a child because we don't know what might be in `bottom`
     // and we need to clear its contents without deleting any object
     // created in the data mask environment
-    data_mask = KEEP(r_new_environment(bottom, 0));
+    data_mask = KEEP(r_alloc_environment(100, bottom));
   }
 
   if (top == r_null) {
@@ -179,9 +179,9 @@ sexp* rlang_new_data_mask(sexp* bottom, sexp* top) {
     check_data_mask_top(bottom, top);
   }
 
-  sexp* ctxt_pronoun = KEEP(rlang_new_ctxt_pronoun(top));
+  r_obj* ctxt_pronoun = KEEP(rlang_new_ctxt_pronoun(top));
 
-  r_env_poke(data_mask, r_tilde_sym, tilde_fn);
+  r_env_poke(data_mask, r_syms.tilde, tilde_fn);
   r_env_poke(data_mask, data_mask_flag_sym, data_mask);
   r_env_poke(data_mask, data_mask_env_sym, ctxt_pronoun);
   r_env_poke(data_mask, data_mask_top_env_sym, top);
@@ -191,17 +191,17 @@ sexp* rlang_new_data_mask(sexp* bottom, sexp* top) {
 }
 
 
-sexp* rlang_is_data_mask(sexp* env) {
+r_obj* ffi_is_data_mask(r_obj* env) {
   return r_lgl(mask_info(env).type == RLANG_MASK_DATA);
 }
 
-static sexp* mask_find(sexp* env, sexp* sym) {
-  if (r_typeof(sym) != r_type_symbol) {
+static r_obj* mask_find(r_obj* env, r_obj* sym) {
+  if (r_typeof(sym) != R_TYPE_symbol) {
     r_abort("Internal error: Data pronoun must be subset with a symbol");
   }
 
-  sexp* top_env = r_env_find(env, data_mask_top_env_sym);
-  if (r_typeof(top_env) == r_type_environment) {
+  r_obj* top_env = r_env_find(env, data_mask_top_env_sym);
+  if (r_typeof(top_env) == R_TYPE_environment) {
     // Start lookup in the parent if the pronoun wraps a data mask
     env = r_env_parent(env);
   } else {
@@ -210,18 +210,18 @@ static sexp* mask_find(sexp* env, sexp* sym) {
     top_env = env;
   }
   int n_kept = 0;
-  KEEP_N(top_env, n_kept);
+  KEEP_N(top_env, &n_kept);
 
-  sexp* cur = env;
+  r_obj* cur = env;
   do {
-    sexp* obj = r_env_find(cur, sym);
-    if (TYPEOF(obj) == PROMSXP) {
+    r_obj* obj = r_env_find(cur, sym);
+    if (r_typeof(obj) == R_TYPE_promise) {
       KEEP(obj);
-      obj = r_eval(obj, r_empty_env);
+      obj = r_eval(obj, r_envs.empty);
       FREE(1);
     }
 
-    if (obj != r_unbound_sym && !r_is_function(obj)) {
+    if (obj != r_syms.unbound) {
       FREE(n_kept);
       return obj;
     }
@@ -231,21 +231,21 @@ static sexp* mask_find(sexp* env, sexp* sym) {
     } else {
       cur = r_env_parent(cur);
     }
-  } while (cur != r_empty_env);
+  } while (cur != r_envs.empty);
 
   FREE(n_kept);
-  return r_unbound_sym;
+  return r_syms.unbound;
 }
-sexp* rlang_data_pronoun_get(sexp* pronoun, sexp* sym) {
-  if (r_typeof(pronoun) != r_type_environment) {
+r_obj* ffi_data_pronoun_get(r_obj* pronoun, r_obj* sym, r_obj* error_call) {
+  if (r_typeof(pronoun) != R_TYPE_environment) {
     r_abort("Internal error: Data pronoun must wrap an environment");
   }
 
-  sexp* obj = mask_find(pronoun, sym);
+  r_obj* obj = mask_find(pronoun, sym);
 
-  if (obj == r_unbound_sym) {
-    sexp* call = KEEP(r_parse("rlang:::abort_data_pronoun(x)"));
-    r_eval_with_x(call, r_base_env, sym);
+  if (obj == r_syms.unbound) {
+    r_obj* call = KEEP(r_parse("abort_data_pronoun(x, call = y)"));
+    r_eval_with_xy(call, sym, error_call, rlang_ns_env);
     r_abort("Internal error: .data subsetting should have failed earlier");
   }
 
@@ -267,54 +267,59 @@ static void warn_env_as_mask_once() {
     "  # Good:\n"
     "  mask <- new_data_mask(env)\n"
     "  eval_tidy(expr, mask)";
-  r_warn_deprecated(msg, msg);
+  warn_deprecated(msg, msg);
 }
 
-static sexp* data_pronoun_sym = NULL;
+static r_obj* data_pronoun_sym = NULL;
+static r_ssize mask_length(r_ssize n);
 
-sexp* rlang_as_data_mask(sexp* data) {
+r_obj* ffi_as_data_mask(r_obj* data) {
   if (mask_info(data).type == RLANG_MASK_DATA) {
     return data;
   }
   if (data == r_null) {
-    return rlang_new_data_mask(r_null, r_null);
+    return ffi_new_data_mask(r_null, r_null);
   }
 
-  int n_protect = 0;
+  int n_kept = 0;
 
-  sexp* bottom = NULL;
+  r_obj* bottom = NULL;
 
   switch (r_typeof(data)) {
-  case r_type_environment:
+  case R_TYPE_environment:
     warn_env_as_mask_once();
-    bottom = KEEP_N(r_env_clone(data, NULL), n_protect);
+    bottom = KEEP_N(r_env_clone(data, NULL), &n_kept);
     break;
 
-  case r_type_logical:
-  case r_type_integer:
-  case r_type_double:
-  case r_type_complex:
-  case r_type_character:
-  case r_type_raw:
-    data = r_vec_coerce(data, r_type_list);
-    KEEP_N(data, n_protect);
+  case R_TYPE_logical:
+  case R_TYPE_integer:
+  case R_TYPE_double:
+  case R_TYPE_complex:
+  case R_TYPE_character:
+  case R_TYPE_raw:
+    data = r_vec_coerce(data, R_TYPE_list);
+    KEEP_N(data, &n_kept);
     // fallthrough:
 
-  case r_type_list: {
+  case R_TYPE_list: {
     check_unique_names(data);
 
-    sexp* names = r_names(data);
-    bottom = KEEP_N(r_new_environment(r_empty_env, 0), n_protect);
+    r_obj* names = r_names(data);
+
+    r_ssize n_mask = mask_length(r_length(data));
+    bottom = KEEP_N(r_alloc_environment(n_mask, r_envs.empty), &n_kept);
 
     if (names != r_null) {
       r_ssize n = r_length(data);
 
+      r_obj* const * p_names = r_chr_cbegin(names);
+      r_obj* const * p_data = r_list_cbegin(data);
+
       for (r_ssize i = 0; i < n; ++i) {
         // Ignore empty or missing names
-        sexp* nm = r_chr_get(names, i);
+        r_obj* nm = p_names[i];
         if (r_str_is_name(nm)) {
-          sexp* elt = r_list_get(data, i);
-          r_env_poke(bottom, r_str_as_symbol(nm), elt);
+          r_env_poke(bottom, r_str_as_symbol(nm), p_data[i]);
         }
       }
     }
@@ -326,28 +331,34 @@ sexp* rlang_as_data_mask(sexp* data) {
     r_abort("`data` must be a vector, list, data frame, or environment");
   }
 
-  sexp* data_mask = KEEP_N(rlang_new_data_mask(bottom, bottom), n_protect);
+  r_obj* data_mask = KEEP_N(ffi_new_data_mask(bottom, bottom), &n_kept);
 
-  sexp* data_pronoun = KEEP_N(rlang_as_data_pronoun(data_mask), n_protect);
+  r_obj* data_pronoun = KEEP_N(ffi_as_data_pronoun(data_mask), &n_kept);
   r_env_poke(bottom, data_pronoun_sym, data_pronoun);
 
-  FREE(n_protect);
+  FREE(n_kept);
   return data_mask;
 }
 
+static
+r_ssize mask_length(r_ssize n) {
+  r_ssize n_grown = r_double_as_ssize(r_double_mult(r_ssize_as_double(n), 1.05));
+  return r_ssize_max(n_grown, r_ssize_add(n, 20));
+}
+
 // For compatibility of the exported C callable
 // TODO: warn
-sexp* rlang_new_data_mask_compat(sexp* bottom, sexp* top, sexp* parent) {
-  return rlang_new_data_mask(bottom, top);
+r_obj* ffi_new_data_mask_compat(r_obj* bottom, r_obj* top, r_obj* parent) {
+  return ffi_new_data_mask(bottom, top);
 }
-sexp* rlang_as_data_mask_compat(sexp* data, sexp* parent) {
-  return rlang_as_data_mask(data);
+r_obj* ffi_as_data_mask_compat(r_obj* data, r_obj* parent) {
+  return ffi_as_data_mask(data);
 }
 
 
-static sexp* tilde_prim = NULL;
+static r_obj* tilde_prim = NULL;
 
-static sexp* base_tilde_eval(sexp* tilde, sexp* quo_env) {
+static r_obj* base_tilde_eval(r_obj* tilde, r_obj* quo_env) {
   if (r_f_has_env(tilde)) {
     return tilde;
   }
@@ -358,19 +369,19 @@ static sexp* base_tilde_eval(sexp* tilde, sexp* quo_env) {
   tilde = KEEP(r_eval(tilde, quo_env));
 
   // Change it back because the result still has the primitive inlined
-  r_node_poke_car(tilde, r_tilde_sym);
+  r_node_poke_car(tilde, r_syms.tilde);
 
   FREE(2);
   return tilde;
 }
 
-sexp* env_get_top_binding(sexp* mask) {
-  sexp* top = r_env_find(mask, data_mask_top_env_sym);
+r_obj* env_get_top_binding(r_obj* mask) {
+  r_obj* top = r_env_find(mask, data_mask_top_env_sym);
 
-  if (top == r_unbound_sym) {
+  if (top == r_syms.unbound) {
     r_abort("Internal error: Can't find .top pronoun in data mask");
   }
-  if (r_typeof(top) != r_type_environment) {
+  if (r_typeof(top) != R_TYPE_environment) {
     r_abort("Internal error: Unexpected .top pronoun type");
   }
 
@@ -378,37 +389,37 @@ sexp* env_get_top_binding(sexp* mask) {
 }
 
 
-static sexp* env_poke_parent_fn = NULL;
-static sexp* env_poke_fn = NULL;
+static r_obj* env_poke_parent_fn = NULL;
+static r_obj* env_poke_fn = NULL;
 
-sexp* rlang_tilde_eval(sexp* tilde, sexp* current_frame, sexp* caller_frame) {
+r_obj* tilde_eval(r_obj* tilde, r_obj* current_frame, r_obj* caller_frame) {
   // Remove srcrefs from system call
-  r_poke_attribute(tilde, r_srcref_sym, r_null);
+  r_attrib_poke(tilde, r_syms.srcref, r_null);
 
-  if (!rlang_is_quosure(tilde)) {
+  if (!is_quosure(tilde)) {
     return base_tilde_eval(tilde, caller_frame);
   }
   if (quo_is_missing(tilde)) {
-    return(r_missing_arg());
+    return(r_missing_arg);
   }
 
-  sexp* expr = rlang_quo_get_expr(tilde);
+  r_obj* expr = quo_get_expr(tilde);
   if (!r_is_symbolic(expr)) {
     return expr;
   }
 
-  sexp* quo_env = rlang_quo_get_env(tilde);
-  if (r_typeof(quo_env) != r_type_environment) {
+  r_obj* quo_env = ffi_quo_get_env(tilde);
+  if (r_typeof(quo_env) != R_TYPE_environment) {
     r_abort("Internal error: Quosure environment is corrupt");
   }
 
-  int n_protect = 0;
-  sexp* top;
+  int n_kept = 0;
+  r_obj* top = r_null;
   struct rlang_mask_info info = mask_info(caller_frame);
 
   switch (info.type) {
   case RLANG_MASK_DATA:
-    top = KEEP_N(env_get_top_binding(info.mask), n_protect);
+    top = KEEP_N(env_get_top_binding(info.mask), &n_kept);
     // Update `.env` pronoun to current quosure env temporarily
     poke_ctxt_env(info.mask, quo_env);
     break;
@@ -422,24 +433,32 @@ sexp* rlang_tilde_eval(sexp* tilde, sexp* current_frame, sexp* caller_frame) {
   // Unless the quosure was created in the mask, swap lexical contexts
   // temporarily by rechaining the top of the mask to the quosure
   // environment
-  if (!r_env_inherits(info.mask, quo_env, top)) {
+  if (!r_env_inherits(quo_env, info.mask, top)) {
     // Unwind-protect the restoration of original parents
     on_exit_restore_lexical_env(info.mask, r_env_parent(top), current_frame);
     r_env_poke_parent(top, quo_env);
   }
 
-  FREE(n_protect);
+  FREE(n_kept);
   return r_eval(expr, info.mask);
 }
 
-static const char* data_mask_objects_names[5] = {
-  ".__tidyeval_data_mask__.", "~", ".top_env", ".env", NULL
+r_obj* ffi_tilde_eval(r_obj* call, r_obj* op, r_obj* args, r_obj* rho) {
+  args = r_node_cdr(args);
+  r_obj* tilde = r_node_car(args); args = r_node_cdr(args);
+  r_obj* current_frame = r_node_car(args); args = r_node_cdr(args);
+  r_obj* caller_frame = r_node_car(args);
+  return tilde_eval(tilde, current_frame, caller_frame);
+}
+
+static const char* data_mask_objects_names[4] = {
+  ".__tidyeval_data_mask__.", "~", ".top_env", ".env"
 };
 
 // Soft-deprecated in rlang 0.2.0
-sexp* rlang_data_mask_clean(sexp* mask) {
-  sexp* bottom = r_env_parent(mask);
-  sexp* top = r_eval(data_mask_top_env_sym, mask);
+r_obj* ffi_data_mask_clean(r_obj* mask) {
+  r_obj* bottom = r_env_parent(mask);
+  r_obj* top = r_eval(data_mask_top_env_sym, mask);
 
   KEEP(top); // Help rchk
 
@@ -448,13 +467,15 @@ sexp* rlang_data_mask_clean(sexp* mask) {
   }
 
   // At this level we only want to remove our own stuff
-  r_env_unbind_strings(mask, data_mask_objects_names);
+  r_env_unbind_c_strings(mask,
+                         data_mask_objects_names,
+                         R_ARR_SIZEOF(data_mask_objects_names));
 
   // Remove everything in the other levels
-  sexp* env = bottom;
-  sexp* parent = r_env_parent(top);
+  r_obj* env = bottom;
+  r_obj* parent = r_env_parent(top);
   while (env != parent) {
-    sexp* nms = KEEP(r_env_names(env));
+    r_obj* nms = KEEP(r_env_names(env));
     r_env_unbind_names(env, nms);
     FREE(1);
     env = r_env_parent(env);
@@ -465,18 +486,18 @@ sexp* rlang_data_mask_clean(sexp* mask) {
 }
 
 
-static sexp* new_quosure_mask(sexp* env) {
-  sexp* mask = KEEP(r_new_environment(env, 3));
-  r_env_poke(mask, r_tilde_sym, tilde_fn);
+static r_obj* new_quosure_mask(r_obj* env) {
+  r_obj* mask = KEEP(r_alloc_environment(3, env));
+  r_env_poke(mask, r_syms.tilde, tilde_fn);
   r_env_poke(mask, quo_mask_flag_sym, mask);
   FREE(1);
   return mask;
 }
 
-sexp* rlang_eval_tidy(sexp* expr, sexp* data, sexp* env) {
-  int n_protect = 0;
+r_obj* rlang_eval_tidy(r_obj* expr, r_obj* data, r_obj* env) {
+  int n_kept = 0;
 
-  if (rlang_is_quosure(expr)) {
+  if (is_quosure(expr)) {
     env = r_quo_get_env(expr);
     expr = r_quo_get_expr(expr);
   }
@@ -485,14 +506,14 @@ sexp* rlang_eval_tidy(sexp* expr, sexp* data, sexp* env) {
   // for quosure thunks. Otherwise we create a heavier data mask with
   // all the masking objects, data pronouns, etc.
   if (data == r_null) {
-    sexp* mask = KEEP_N(new_quosure_mask(env), n_protect);
-    sexp* out = r_eval(expr, mask);
-    FREE(n_protect);
+    r_obj* mask = KEEP_N(new_quosure_mask(env), &n_kept);
+    r_obj* out = r_eval(expr, mask);
+    FREE(n_kept);
     return out;
   }
 
-  sexp* mask = KEEP_N(rlang_as_data_mask(data), n_protect);
-  sexp* top = KEEP_N(env_get_top_binding(mask), n_protect);
+  r_obj* mask = KEEP_N(ffi_as_data_mask(data), &n_kept);
+  r_obj* top = KEEP_N(env_get_top_binding(mask), &n_kept);
 
   // Rechain the mask on the new lexical env but don't restore it on
   // exit. This way leaked masks inherit from a somewhat sensible
@@ -509,42 +530,40 @@ sexp* rlang_eval_tidy(sexp* expr, sexp* data, sexp* env) {
   // * Look-up in leaked environments would proceed from the data mask
   //   to the appropriate lexical environment (from quosures or from
   //   the `env` argument of eval_tidy()).
-  if (!r_env_inherits(mask, env, top)) {
+  if (!r_env_inherits(env, mask, top)) {
     poke_ctxt_env(mask, env);
     r_env_poke_parent(top, env);
   }
 
-  sexp* out = r_eval(expr, mask);
-  FREE(n_protect);
+  r_obj* out = r_eval(expr, mask);
+  FREE(n_kept);
   return out;
 }
 
+r_obj* ffi_eval_tidy(r_obj* call, r_obj* op, r_obj* args, r_obj* rho) {
+  args = r_node_cdr(args);
+  r_obj* expr = r_node_car(args); args = r_node_cdr(args);
+  r_obj* data = r_node_car(args); args = r_node_cdr(args);
+  r_obj* env = r_node_car(args);
+  return rlang_eval_tidy(expr, data, env);
+}
+
 
 void rlang_init_eval_tidy() {
-  sexp* rlang_ns_env = KEEP(r_ns_env("rlang"));
+  r_obj* rlang_ns_env = KEEP(r_ns_env("rlang"));
 
-  tilde_fn = r_parse_eval(
-    "function(...) {                          \n"
-    "  .Call(rlang_tilde_eval,                \n"
-    "    sys.call(),     # Quosure env        \n"
-    "    environment(),  # Unwind-protect env \n"
-    "    parent.frame()  # Lexical env        \n"
-    "  )                                      \n"
-    "}                                        \n",
-    rlang_ns_env
-  );
-  r_mark_precious(tilde_fn);
+  tilde_fn = r_eval(r_sym("tilde_eval"), rlang_ns_env);
 
   data_pronoun_class = r_chr("rlang_data_pronoun");
-  r_mark_precious(data_pronoun_class);
+  r_preserve(data_pronoun_class);
 
   ctxt_pronoun_class = r_chr("rlang_ctxt_pronoun");
-  r_mark_precious(ctxt_pronoun_class);
+  r_preserve(ctxt_pronoun_class);
 
-  empty_names_chr = r_new_vector(r_type_character, 2);
-  r_mark_precious(empty_names_chr);
-  r_chr_poke(empty_names_chr, 0, r_string(""));
-  r_chr_poke(empty_names_chr, 1, r_missing_str);
+  empty_names_chr = r_alloc_character(2);
+  r_preserve(empty_names_chr);
+  r_chr_poke(empty_names_chr, 0, r_str(""));
+  r_chr_poke(empty_names_chr, 1, r_globals.na_str);
 
   quo_mask_flag_sym = r_sym(".__tidyeval_quosure_mask__.");
   data_mask_flag_sym = r_sym(".__tidyeval_data_mask__.");
@@ -574,9 +593,9 @@ void rlang_init_eval_tidy() {
     "                                      \n"
     "  parent.env(top) <- `old`            \n"
     "}                                     \n",
-    r_base_env
+    r_envs.base
   );
-  r_mark_precious(restore_mask_fn);
+  r_preserve(restore_mask_fn);
 
   FREE(1);
 }
