@@ -69,6 +69,7 @@ static const R_CallMethodDef r_callables[] = {
   {"ffi_env_binding_types",            (DL_FUNC) &r_env_binding_types, 2},
   {"ffi_env_browse",                   (DL_FUNC) &ffi_env_browse, 2},
   {"ffi_env_clone",                    (DL_FUNC) &r_env_clone, 2},
+  {"ffi_env_coalesce",                 (DL_FUNC) &ffi_env_coalesce, 2},
   {"ffi_env_frame",                    (DL_FUNC) &ffi_env_frame, 1},
   {"ffi_env_get",                      (DL_FUNC) &ffi_env_get, 5},
   {"ffi_env_get_list",                 (DL_FUNC) &ffi_env_get_list, 5},
@@ -89,6 +90,7 @@ static const R_CallMethodDef r_callables[] = {
   {"ffi_fini_rlang",                   (DL_FUNC) &ffi_fini_rlang, 0},
   {"ffi_format_error_arg",             (DL_FUNC) &ffi_format_error_arg, 1},
   {"ffi_get_expression",               (DL_FUNC) &ffi_get_expression, 2},
+  {"ffi_getppid",                      (DL_FUNC) &ffi_getppid, 0},
   {"ffi_glue_is_here",                 (DL_FUNC) &ffi_glue_is_here, 0},
   {"ffi_has_local_precious_list",      (DL_FUNC) &ffi_has_local_precious_list, 0},
   {"ffi_hash",                         (DL_FUNC) &ffi_hash, 1},
@@ -212,6 +214,7 @@ static const R_CallMethodDef r_callables[] = {
   {"ffi_test_parse_eval",              (DL_FUNC) &ffi_test_parse_eval, 2},
   {"ffi_test_r_on_exit",               (DL_FUNC) &r_on_exit, 2},
   {"ffi_test_r_warn",                  (DL_FUNC) &ffi_test_r_warn, 1},
+  {"ffi_test_stop_internal",           (DL_FUNC) &ffi_test_stop_internal, 1},
   {"ffi_test_sys_call",                (DL_FUNC) &ffi_test_sys_call, 1},
   {"ffi_test_sys_frame",               (DL_FUNC) &ffi_test_sys_frame, 1},
   {"ffi_true_length",                  (DL_FUNC) &ffi_true_length, 1},
@@ -233,7 +236,7 @@ static const R_CallMethodDef r_callables[] = {
 
 
 static const R_ExternalMethodDef externals[] = {
-  {"ffi_arg_match0",                    (DL_FUNC) &ffi_arg_match0, 4},
+  {"ffi_arg_match0",                    (DL_FUNC) &ffi_arg_match0, 3},
   {"ffi_call2",                         (DL_FUNC) &ffi_call2, 2},
   {"ffi_capturearginfo",                (DL_FUNC) &ffi_capturearginfo, 2},
   {"ffi_capturedots",                   (DL_FUNC) &ffi_capturedots, 1},
@@ -259,47 +262,50 @@ extern uint64_t XXH3_64bits(const void*, size_t);
 
 r_visible
 void R_init_rlang(DllInfo* dll) {
-  R_RegisterCCallable("rlang", "rlang_arg_match",           (DL_FUNC) &arg_match);
-  R_RegisterCCallable("rlang", "rlang_as_data_mask_3.0.0",  (DL_FUNC) &ffi_as_data_mask);
-  R_RegisterCCallable("rlang", "rlang_as_data_pronoun",     (DL_FUNC) &ffi_as_data_pronoun);
-  R_RegisterCCallable("rlang", "rlang_env_unbind",          (DL_FUNC) &r_env_unbind);
-  R_RegisterCCallable("rlang", "rlang_eval_tidy",           (DL_FUNC) &rlang_eval_tidy);
-  R_RegisterCCallable("rlang", "rlang_format_error_arg",    (DL_FUNC) &rlang_format_error_arg);
-  R_RegisterCCallable("rlang", "rlang_is_quosure",          (DL_FUNC) &is_quosure);
-  R_RegisterCCallable("rlang", "rlang_names_as_unique",     (DL_FUNC) &names_as_unique);
-  R_RegisterCCallable("rlang", "rlang_new_data_mask_3.0.0", (DL_FUNC) &ffi_new_data_mask);
-  R_RegisterCCallable("rlang", "rlang_new_quosure",         (DL_FUNC) &ffi_new_quosure);
-  R_RegisterCCallable("rlang", "rlang_quo_get_env",         (DL_FUNC) &ffi_quo_get_env);
-  R_RegisterCCallable("rlang", "rlang_quo_get_expr",        (DL_FUNC) &ffi_quo_get_expr);
-  R_RegisterCCallable("rlang", "rlang_quo_set_env",         (DL_FUNC) &ffi_quo_set_env);
-  R_RegisterCCallable("rlang", "rlang_quo_set_expr",        (DL_FUNC) &ffi_quo_set_expr);
-  R_RegisterCCallable("rlang", "rlang_stop_internal",       (DL_FUNC) &rlang_stop_internal);
-  R_RegisterCCallable("rlang", "rlang_stop_internal2",      (DL_FUNC) &rlang_stop_internal2);
+  R_RegisterCCallable("rlang", "rlang_arg_match",              (DL_FUNC) &arg_match_legacy);
+  R_RegisterCCallable("rlang", "rlang_arg_match_2",            (DL_FUNC) &cci_arg_match);
+  R_RegisterCCallable("rlang", "rlang_as_data_mask_3.0.0",     (DL_FUNC) &ffi_as_data_mask);
+  R_RegisterCCallable("rlang", "rlang_as_data_pronoun",        (DL_FUNC) &ffi_as_data_pronoun);
+  R_RegisterCCallable("rlang", "rlang_env_unbind",             (DL_FUNC) &r_env_unbind);
+  R_RegisterCCallable("rlang", "rlang_eval_tidy",              (DL_FUNC) &rlang_eval_tidy);
+  R_RegisterCCallable("rlang", "rlang_format_error_arg",       (DL_FUNC) &rlang_format_error_arg);
+  R_RegisterCCallable("rlang", "rlang_is_quosure",             (DL_FUNC) &is_quosure);
+  R_RegisterCCallable("rlang", "rlang_names_as_unique",        (DL_FUNC) &names_as_unique);
+  R_RegisterCCallable("rlang", "rlang_new_data_mask_3.0.0",    (DL_FUNC) &ffi_new_data_mask);
+  R_RegisterCCallable("rlang", "rlang_new_quosure",            (DL_FUNC) &ffi_new_quosure);
+  R_RegisterCCallable("rlang", "rlang_obj_type_friendly_full", (DL_FUNC) &rlang_obj_type_friendly_full);
+  R_RegisterCCallable("rlang", "rlang_quo_get_env",            (DL_FUNC) &ffi_quo_get_env);
+  R_RegisterCCallable("rlang", "rlang_quo_get_expr",           (DL_FUNC) &ffi_quo_get_expr);
+  R_RegisterCCallable("rlang", "rlang_quo_set_env",            (DL_FUNC) &ffi_quo_set_env);
+  R_RegisterCCallable("rlang", "rlang_quo_set_expr",           (DL_FUNC) &ffi_quo_set_expr);
+  R_RegisterCCallable("rlang", "rlang_stop_internal",          (DL_FUNC) &rlang_stop_internal);
+  R_RegisterCCallable("rlang", "rlang_stop_internal2",         (DL_FUNC) &rlang_stop_internal2);
 
   r_obj* r_as_function(r_obj* x, const char* arg);
-  R_RegisterCCallable("rlang", "rlang_as_function",         (DL_FUNC) &r_as_function);
+  R_RegisterCCallable("rlang", "rlang_as_function",            (DL_FUNC) &r_as_function);
 
-  R_RegisterCCallable("rlang", "rlang_xxh3_64bits",         (DL_FUNC) &XXH3_64bits);
+  R_RegisterCCallable("rlang", "rlang_xxh3_64bits",            (DL_FUNC) &XXH3_64bits);
 
   // Maturing
-  R_RegisterCCallable("rlang", "rlang_env_dots_list",       (DL_FUNC) &rlang_env_dots_list);
-  R_RegisterCCallable("rlang", "rlang_env_dots_values",     (DL_FUNC) &rlang_env_dots_values);
-  R_RegisterCCallable("rlang", "rlang_is_splice_box",       (DL_FUNC) &is_splice_box);
-  R_RegisterCCallable("rlang", "rlang_obj_encode_utf8",     (DL_FUNC) &obj_encode_utf8);
-  R_RegisterCCallable("rlang", "rlang_str_as_symbol",       (DL_FUNC) &r_str_as_symbol);
-  R_RegisterCCallable("rlang", "rlang_sym_as_character",    (DL_FUNC) &ffi_sym_as_character);
-  R_RegisterCCallable("rlang", "rlang_sym_as_string",       (DL_FUNC) &ffi_sym_as_string);
-  R_RegisterCCallable("rlang", "rlang_unbox",               (DL_FUNC) &rlang_unbox);
+  R_RegisterCCallable("rlang", "rlang_env_dots_list",          (DL_FUNC) &rlang_env_dots_list);
+  R_RegisterCCallable("rlang", "rlang_env_dots_values",        (DL_FUNC) &rlang_env_dots_values);
+  R_RegisterCCallable("rlang", "rlang_is_splice_box",          (DL_FUNC) &is_splice_box);
+  R_RegisterCCallable("rlang", "rlang_obj_encode_utf8",        (DL_FUNC) &obj_encode_utf8);
+  R_RegisterCCallable("rlang", "rlang_str_as_symbol",          (DL_FUNC) &r_str_as_symbol);
+  R_RegisterCCallable("rlang", "rlang_sym_as_character",       (DL_FUNC) &ffi_sym_as_character);
+  R_RegisterCCallable("rlang", "rlang_sym_as_string",          (DL_FUNC) &ffi_sym_as_string);
+  R_RegisterCCallable("rlang", "rlang_unbox",                  (DL_FUNC) &rlang_unbox);
 
   // Experimental
-  R_RegisterCCallable("rlang", "rlang_squash_if",           (DL_FUNC) &r_squash_if);
+  R_RegisterCCallable("rlang", "rlang_squash_if",              (DL_FUNC) &r_squash_if);
 
   // Compatibility
-  R_RegisterCCallable("rlang", "rlang_as_data_mask",        (DL_FUNC) &ffi_as_data_mask_compat);
-  R_RegisterCCallable("rlang", "rlang_new_data_mask",       (DL_FUNC) &ffi_new_data_mask_compat);
+  R_RegisterCCallable("rlang", "rlang_as_data_mask",           (DL_FUNC) &ffi_as_data_mask_compat);
+  R_RegisterCCallable("rlang", "rlang_new_data_mask",          (DL_FUNC) &ffi_new_data_mask_compat);
 
   // Only for debugging - no stability guaranteed
-  R_RegisterCCallable("rlang", "rlang_print_backtrace",     (DL_FUNC) &rlang_print_backtrace);
+  R_RegisterCCallable("rlang", "rlang_print_backtrace",        (DL_FUNC) &rlang_print_backtrace);
+  R_RegisterCCallable("rlang", "rlang_env_print",              (DL_FUNC) &rlang_env_print);
 
   R_registerRoutines(dll, NULL, r_callables, NULL, externals);
   R_useDynamicSymbols(dll, FALSE);
