@@ -95,15 +95,6 @@ test_that("corner cases are handled when interpolating dot names", {
     expect_error(quos(!!var := NULL), "must be a string or a symbol")
 })
 
-test_that("definitions are interpolated", {
-  var1 <- "foo"
-  var2 <- "bar"
-  dots <- dots_definitions(def = foo(!!var1) := bar(!!var2))
-
-  pat <- list(lhs = quo(foo("foo")), rhs = quo(bar("bar")))
-  expect_identical(dots$defs$def, pat)
-})
-
 test_that("dots are forwarded to named arguments", {
   outer <- function(...) inner(...)
   inner <- function(...) fn(...)
@@ -236,7 +227,7 @@ test_that("serialised unicode in `:=` LHS is unserialised", {
     exprs <- exprs("\u5e78" := 10)
     names(exprs)
   })
-  expect_identical(as_bytes(nms), as_bytes("\u5e78"))
+  expect_identical(charToRaw(nms), charToRaw("\u5e78"))
 })
 
 test_that("exprs() supports auto-naming", {
@@ -397,7 +388,7 @@ test_that("endots() supports `.ignore_empty`", {
 test_that("ensyms() captures multiple symbols", {
   fn <- function(arg, ...) ensyms(arg, ...)
   expect_identical(fn(foo, bar, baz), exprs(foo, bar, baz))
-  expect_error(fn(foo()), "Only strings can be converted to symbols")
+  expect_snapshot(err(fn(foo())))
 })
 
 test_that("enquos() works with lexically scoped dots", {
@@ -414,20 +405,6 @@ test_that("enquo() works with lexically scoped arguments", {
   expect_identical(capture(foo), quo(foo))
 })
 
-test_that("dots_definitions() uses tidy eval", {
-  var1 <- "foo"
-  var2 <- "bar"
-  dots <- dots_definitions(pat = foo(!!var1) := bar(!!var2))
-  pat <- list(lhs = quo(foo("foo")), rhs = quo(bar("bar")))
-  expect_identical(dots$defs$pat, pat)
-})
-
-test_that("dots_definitions() accepts other types of arguments", {
-  dots <- dots_definitions(A = a := b, B = c)
-  expect_identical(dots$defs$A, list(lhs = quo(a), rhs = quo(b)))
-  expect_identical(dots$dots$B, quo(c))
-})
-
 test_that("closures are captured with their calling environment", {
   expect_reference(quo_get_env(quo(!!function() NULL)), environment())
 })
@@ -457,10 +434,6 @@ test_that("missing names are forwarded", {
   expect_identical_(names(exprs(!!!x)), chr(na_chr, na_chr))
 })
 
-test_that("`.named` must be integerish", {
-  expect_error(exprs(foo, .named = 100.5), "must be a scalar logical")
-})
-
 test_that("auto-naming uses type_sum() (#573)", {
   expect_named(quos(foo, !!(1:3), .named = TRUE), c("foo", "<int>"))
 
@@ -485,6 +458,18 @@ test_that("enexprs() and enquos() support `.ignore_empty = 'all'` (#414)", {
   expect_identical(myquos("all"), quos())
 })
 
+test_that("`enquos()` does not discard named missing arguments (#1229)", {
+  fn <- function(...) enquos(..., .ignore_empty = "all")
+  expect_equal(
+    fn(x = ),
+    quos(x = )
+  )
+  expect_equal(
+    fn(, foo),
+    quos(foo)
+  )
+})
+
 test_that("enexprs() and enquos() support empty dots", {
   myexprs <- function(what, ...) enexprs(..., .ignore_empty = what)
   expect_identical(myexprs("none"), exprs())
@@ -507,13 +492,13 @@ test_that("ensym() unwraps quosures", {
   fn <- function(arg) ensym(arg)
   expect_identical(fn(!!quo(foo)), quote(foo))
   expect_identical(fn(!!quo("foo")), quote(foo))
-  expect_error(fn(!!quo(foo())), "Only strings can be converted to symbols")
+  expect_snapshot(err(fn(!!quo(foo()))))
 })
 
 test_that("ensyms() unwraps quosures", {
   fn <- function(...) ensyms(...)
   expect_identical(fn(!!!quos(foo, "bar")), exprs(foo, bar))
-  expect_error(fn(!!!quos(foo, bar())), "Only strings can be converted to symbols")
+  expect_snapshot(err(fn(!!!quos(foo, bar()))))
 })
 
 test_that("enquo0() and enquos0() capture arguments without injection", {
@@ -539,3 +524,101 @@ test_that("enquo0() and enquos0() don't rewrap quosures", {
   quo <- local(quo(x))
   expect_equal(fn(!!quo), list(quo))
 })
+
+test_that("enquo() defuses numbered dots (#1137)", {
+  f <- function(arg) enquo(..1)
+  expect_error(
+    f(foo),
+    "'...' used in an incorrect context"
+  )
+
+  f <- function(...) enquo(..1)
+  expect_error(
+    f(),
+    "fewer than 1"
+  )
+
+  f <- function(...) enquo(..2)
+  expect_error(
+    f(1),
+    "fewer than 2"
+  )
+})
+
+test_that("enquos() defuses numbered dots (#1137)", {
+  f <- function(...) enquos(...)
+  g <- function(...) f(..1)
+  expect_equal(
+    g(foo),
+    quos(foo)
+  )
+
+  f <- function(...) enquos(...)
+  g <- function(...) f(..1, ..2)
+  h <- function(...) g(..1, ...)
+  expect_equal(
+    h(foo, bar),
+    quos(foo, foo)
+  )
+
+  g <- function(...) f(..1, ..3)
+  expect_equal(
+    h(foo, bar),
+    quos(foo, bar)
+  )
+
+  g <- function(...) f(..1, ..4)
+  expect_error(
+    h(foo, bar),
+    "fewer than 4 elements"
+  )
+})
+
+test_that("`defer()` does not crash with environments containing quosures (#1085)", {
+  f <- function() {
+    withr::defer(2)
+    dots <- quos(integer(1))
+    quo(c(!!!dots))
+  }
+  expect_no_error(f()) # No crash
+})
+
+test_that("auto-named expressions can be unique-repaired", {
+  dots_names <- function(...) {
+    dots <- enquos(...)
+    dots <- exprs_auto_name(dots, repair_auto = "unique")
+    names(dots)
+  }
+
+  expect_snapshot({
+    expect_equal(
+      dots_names(1, foo = 1, 1, foo = 2),
+      c("1...1", "foo", "1...3", "foo")
+    )
+
+    expect_equal(
+      dots_names(bar, foo = 1, bar, foo = 2),
+      c("bar...1", "foo", "bar...3", "foo")
+    )
+  })
+})
+
+test_that("can capture forced numbered dot", {
+  fn <- function(..., x = ..1) {
+    force(x)
+    enquo(x)
+  }
+  expect_equal(fn(1 + 1), quo(2))
+})
+
+test_that("`enexprs()` and variants support `.named = NULL` (#1223)", {
+  fn <- function(...) enexprs(..., .named = NULL)
+  expect_equal(fn(), list())
+  expect_equal(fn(1), list(1))
+  expect_equal(fn(x = 1), list(x = 1))
+
+  fn <- function(...) enquos(..., .named = NULL)
+  expect_equal(fn(), unname(quos()))
+  expect_equal(fn(1), unname(quos(1)))
+  expect_equal(fn(x = 1), quos(x = 1))
+})
