@@ -1,12 +1,15 @@
-context("trace.R")
+local_options(
+  rlang_trace_use_winch = FALSE
+)
 
 # These tests must come first because print method includes srcrefs
 test_that("tree printing only changes deliberately", {
-  skip_unless_utf8()
+  # Because of srcrefs
+  skip_on_cran()
+  skip_if_not_installed("testthat", "2.99.0")
 
   local_options(
-    rlang_trace_format_srcrefs = TRUE,
-    `rlang:::trace_force_dangling_srcrefs` = TRUE
+    rlang_trace_format_srcrefs = TRUE
   )
 
   dir <- normalizePath(test_path(".."))
@@ -21,22 +24,23 @@ test_that("tree printing only changes deliberately", {
   l <- function(i) trace_back(e)
   trace <- i()
 
-  expect_known_output(file = test_path("test-trace-print.txt"), {
+  expect_snapshot({
     print(trace, dir = dir)
     cat("\n")
-    print(trace_subset(trace, 0L), dir = dir)
+    print(trace_slice(trace, 0L), dir = dir)
   })
 })
 
 test_that("can print tree with collapsed branches", {
-  skip_unless_utf8()
+  # Because of srcrefs
+  skip_on_cran()
+  skip_if_not_installed("testthat", "2.99.0")
 
   # Fake eval() call does not have same signature on old R
   skip_if(getRversion() < "3.4")
 
   local_options(
-    rlang_trace_format_srcrefs = TRUE,
-    `rlang:::trace_force_dangling_srcrefs` = TRUE
+    rlang_trace_format_srcrefs = TRUE
   )
 
   dir <- normalizePath(test_path(".."))
@@ -48,8 +52,7 @@ test_that("can print tree with collapsed branches", {
   i <- function() { tryCatch(trace_back(e, bottom = 0)) }
   trace <- eval(quote(f()))
 
-  expect_known_trace_output(trace,
-    file = test_path("test-trace-collapsed1.txt"),
+  expect_snapshot_trace(trace,
     dir = dir,
     srcrefs = TRUE
   )
@@ -60,8 +63,7 @@ test_that("can print tree with collapsed branches", {
   h <- function() trace_back(e)
   trace <- eval(quote(f()))
 
-  expect_known_trace_output(trace,
-    file = test_path("test-trace-collapsed2.txt"),
+  expect_snapshot_trace(trace,
     dir = dir,
     srcrefs = TRUE
   )
@@ -75,21 +77,21 @@ test_that("trace_simplify_branch() extracts last branch", {
   m <- function() trace_back(e)
 
   x1 <- j(1)
-  expect_trace_length(x1, 6)
-  expect_trace_length(trace_simplify_branch(x1), 3)
+  expect_equal(sum(x1$visible), 6)
+  expect_equal(sum(trace_simplify_branch(x1)$visible), 3)
 
   x2 <- j(2)
-  expect_trace_length(x2, 6)
-  expect_trace_length(trace_simplify_branch(x2), 2)
+  expect_equal(sum(x2$visible), 6)
+  expect_equal(sum(trace_simplify_branch(x2)$visible), 2)
 
   x3 <- j(3)
-  expect_trace_length(x2, 6)
-  expect_trace_length(trace_simplify_branch(x3), 1)
+  expect_equal(sum(x3$visible), 1)
+  expect_equal(sum(trace_simplify_branch(x3)$visible), 1)
 })
 
 test_that("integerish indices are allowed", {
   trace <- trace_back()
-  expect_identical(trace_subset(trace, 0), trace_subset(trace, 0L))
+  expect_identical(trace_slice(trace, 0), trace_slice(trace, 0L))
 })
 
 test_that("cli_branch() handles edge case", {
@@ -97,14 +99,11 @@ test_that("cli_branch() handles edge case", {
   f <- function() trace_back(e)
   trace <- f()
 
-  call <- paste0(" ", cli_style$h, "rlang:::f()")
   tree <- trace_as_tree(trace, srcrefs = FALSE)
-  expect_identical(cli_branch(tree$call[-1], trace$indices), call)
+  expect_snapshot(cli_branch(tree[-1, ]))
 })
 
 test_that("trace formatting picks up `rlang_trace_format_srcrefs`", {
-  local_options(`rlang:::trace_force_dangling_srcrefs` = TRUE)
-
   e <- environment()
   f <- function() trace_back(e)
   trace <- f()
@@ -128,22 +127,20 @@ test_that("trace picks up option `rlang_trace_top_env` for trimming trace", {
   )
 })
 
+# This test used to be about `simplify = "collapse"`
 test_that("collapsed formatting doesn't collapse single frame siblings", {
   e <- current_env()
   f <- function() eval_bare(quote(g()))
   g <- function() trace_back(e)
   trace <- f()
 
-  full <- capture.output(print(trace, simplify = "none", srcrefs = FALSE))[[3]]
-  expect_match(full, "rlang::eval_bare(quote(g()))", fixed = TRUE)
-
-  collapsed <- capture.output(print(trace, simplify = "collapse", srcrefs = FALSE))[[3]]
-  expect_match(collapsed, "[ rlang::eval_bare(...) ]", fixed = TRUE)
+  expect_snapshot({
+    print(trace, simplify = "none", drop = TRUE, srcrefs = FALSE)
+    print(trace, simplify = "none", drop = FALSE, srcrefs = FALSE)
+  })
 })
 
 test_that("recursive frames are rewired to the global env", {
-  skip_unless_utf8()
-
   dir <- normalizePath(test_path(".."))
   e <- environment()
 
@@ -151,12 +148,10 @@ test_that("recursive frames are rewired to the global env", {
   g <- function() trace_back(e)
   trace <- eval_tidy(quo(f()))
 
-  expect_known_trace_output(trace, file = "test-trace-recursive.txt")
+  expect_snapshot_trace(trace)
 })
 
 test_that("long backtrace branches are truncated", {
-  skip_unless_utf8()
-
   e <- current_env()
   f <- function(n) {
     if (n) {
@@ -166,7 +161,7 @@ test_that("long backtrace branches are truncated", {
   }
   trace <- f(10)
 
-  expect_known_output(file = test_path("test-trace-truncate-backtrace-branch.txt"), {
+  expect_snapshot({
     cat("Full:\n")
     print(trace, simplify = "branch", srcrefs = FALSE)
     cat("\n5 frames:\n")
@@ -178,12 +173,9 @@ test_that("long backtrace branches are truncated", {
   })
 
   expect_error(print(trace, simplify = "none", max_frames = 5), "currently only supported with")
-  expect_error(print(trace, simplify = "collapse", max_frames = 5), "currently only supported with")
 })
 
 test_that("eval() frames are collapsed", {
-  skip_unless_utf8()
-
   # Fake eval() call does not have same signature on old R
   skip_if(getRversion() < "3.4")
 
@@ -192,62 +184,16 @@ test_that("eval() frames are collapsed", {
   g <- function() eval(quote(trace_back(e, bottom = 0)))
   trace <- f()
 
-  expect_known_trace_output(trace, file = "test-trace-collapse-eval.txt")
+  expect_snapshot_trace(trace)
 
   f <- function() base::evalq(g())
   g <- function() evalq(trace_back(e, bottom = 0))
   trace <- f()
 
-  expect_known_trace_output(trace, file = "test-trace-collapse-evalq.txt")
-})
-
-test_that("%>% frames are collapsed", {
-  skip_unless_utf8()
-  skip_if_not_installed("magrittr")
-
-  # Fake eval() call does not have same signature on old R
-  skip_if(getRversion() < "3.4")
-
-  `%>%` <- magrittr::`%>%`
-
-  e <- current_env()
-  f <- function(x, ...) x
-  g <- function(x, ...) x
-  h <- function(x, ...) trace_back(e)
-
-  trace <- NULL %>% f() %>% g(1, 2) %>% h(3, ., 4)
-  expect_known_trace_output(trace, "test-trace-collapse-magrittr.txt")
-
-  trace <- f(NULL) %>% g(list(.)) %>% h(3, ., list(.))
-  expect_known_trace_output(trace, "test-trace-collapse-magrittr2.txt")
-
-  trace <- f(g(NULL %>% f()) %>% h())
-  expect_known_trace_output(trace, "test-trace-collapse-magrittr3.txt")
-})
-
-test_that("children of collapsed %>% frames have correct parent", {
-  skip_unless_utf8()
-  skip_if_not_installed("magrittr")
-
-  # Fake eval() call does not have same signature on old R
-  skip_if(getRversion() < "3.4")
-
-  `%>%` <- magrittr::`%>%`
-
-  e <- current_env()
-  F <- function(x, ...) x
-  G <- function(x, ...) x
-  H <- function(x) f()
-  f <- function() h()
-  h <- function() trace_back(e)
-
-  trace <- NA %>% F() %>% G() %>% H()
-  expect_known_trace_output(trace, "test-trace-collapse-magrittr-children.txt")
+  expect_snapshot_trace(trace)
 })
 
 test_that("children of collapsed frames are rechained to correct parent", {
-  skip_unless_utf8()
-
   # Fake eval() call does not have same signature on old R
   skip_if(getRversion() < "3.4")
 
@@ -256,39 +202,19 @@ test_that("children of collapsed frames are rechained to correct parent", {
   g <- function() trace_back(e)
   trace <- f()
 
-  expect_known_output(file = test_path("test-trace-collapse-children.txt"), {
-    cat("Full:\n")
-    print(trace, simplify = "none", srcrefs = FALSE)
-    cat("\nCollapsed:\n")
-    print(trace, simplify = "collapse", srcrefs = FALSE)
+  expect_snapshot({
+    cat("Full + drop:\n")
+    print(trace, simplify = "none", drop = TRUE, srcrefs = FALSE)
+    cat("Full - drop:\n")
+    print(trace, simplify = "none", drop = FALSE, srcrefs = FALSE)
     cat("\nBranch:\n")
     print(trace, simplify = "branch", srcrefs = FALSE)
   })
 })
 
-test_that("pipe_collect_calls() collects calls", {
-  exprs2 <- function(...) unname(exprs(...))
-  placeholder <- function() NULL
-
-  call <- quote(a(A %>% B) %>% b)
-  out <- pipe_collect_calls(call, placeholder)
-  expect_identical(out$calls, exprs2(rlang:::a(A %>% B), rlang:::b(.)))
-  expect_true(out$leading)
-
-  call <- quote(a %>% b %>% c)
-  out <- pipe_collect_calls(call, placeholder)
-  expect_identical(out$calls, exprs2(rlang:::b(.), rlang:::c(.)))
-  expect_false(out$leading)
-
-  call <- quote(a() %>% b %>% c)
-  out <- pipe_collect_calls(call, placeholder)
-  expect_identical(out$calls, exprs2(rlang:::a(), rlang:::b(.), rlang:::c(.)))
-  expect_true(out$leading)
-})
-
 test_that("combinations of incomplete and leading pipes collapse properly", {
-  skip_unless_utf8()
-  skip_if_not_installed("magrittr")
+  skip_if_not_installed("magrittr", "1.5.0.9000")
+  skip_on_cran()
 
   # Fake eval() call does not have same signature on old R
   skip_if(getRversion() < "3.4")
@@ -300,30 +226,30 @@ test_that("combinations of incomplete and leading pipes collapse properly", {
   T <- function(x) trace_back(e)
 
   trace <- NA %>% F() %>% T() %>% F() %>% F()
-  expect_known_trace_output(trace, "test-trace-collapse-magrittr-incomplete.txt")
+  expect_snapshot_trace(trace)
 
   trace <- T(NA) %>% F()
-  expect_known_trace_output(trace, "test-trace-collapse-magrittr-incomplete-leading1.txt")
+  expect_snapshot_trace(trace)
 
   trace <- F(NA) %>% F() %>% T() %>% F() %>% F()
-  expect_known_trace_output(trace, "test-trace-collapse-magrittr-incomplete-leading2.txt")
+  expect_snapshot_trace(trace)
 
   trace <- NA %>% T()
-  expect_known_trace_output(trace, "test-trace-collapse-magrittr-complete1.txt")
+  expect_snapshot_trace(trace)
 
   trace <- NA %>% F() %>% T()
-  expect_known_trace_output(trace, "test-trace-collapse-magrittr-complete2.txt")
+  expect_snapshot_trace(trace)
 
   trace <- F(NA) %>% T()
-  expect_known_trace_output(trace, "test-trace-collapse-magrittr-complete-leading1.txt")
+  expect_snapshot_trace(trace)
 
   trace <- F(NA) %>% F() %>%  T()
-  expect_known_trace_output(trace, "test-trace-collapse-magrittr-complete-leading2.txt")
+  expect_snapshot_trace(trace)
 })
 
 test_that("calls before and after pipe are preserved", {
-  skip_unless_utf8()
-  skip_if_not_installed("magrittr")
+  skip_if_not_installed("magrittr", "1.5.0.9000")
+  skip_on_cran()
 
   # Fake eval() call does not have same signature on old R
   skip_if(getRversion() < "3.4")
@@ -337,18 +263,16 @@ test_that("calls before and after pipe are preserved", {
   f <- function() trace_back(e)
 
   trace <- F(NA %>% T())
-  expect_known_trace_output(trace, "test-trace-collapse-magrittr-before-after1.txt")
+  expect_snapshot_trace(trace)
 
   trace <- NA %>% C()
-  expect_known_trace_output(trace, "test-trace-collapse-magrittr-before-after2.txt")
+  expect_snapshot_trace(trace)
 
   trace <- F(NA %>% C())
-  expect_known_trace_output(trace, "test-trace-collapse-magrittr-before-after3.txt")
+  expect_snapshot_trace(trace)
 })
 
 test_that("always keep very first frame as part of backtrace branch", {
-  skip_unless_utf8()
-
   # Fake eval() call does not have same signature on old R
   skip_if(getRversion() < "3.4")
 
@@ -358,7 +282,7 @@ test_that("always keep very first frame as part of backtrace branch", {
   gen.default <- function(x) trace_back(e)
 
   trace <- gen()
-  expect_known_trace_output(trace, "test-trace-backtrace-branch-first-frame.txt")
+  expect_snapshot_trace(trace)
 })
 
 test_that("can take the str() of a trace (#615)", {
@@ -368,8 +292,6 @@ test_that("can take the str() of a trace (#615)", {
 })
 
 test_that("anonymous calls are stripped from backtraces", {
-  skip_unless_utf8()
-
   e <- current_env()
   trace <- (function() {
     "foo"
@@ -378,12 +300,10 @@ test_that("anonymous calls are stripped from backtraces", {
   })()
 
   expect_identical(format(trace, simplify = "branch"), chr())
-  expect_known_trace_output(trace, "test-trace-backtrace-anonymous.txt")
+  expect_snapshot_trace(trace)
 })
 
 test_that("collapsing of eval() frames detects when error occurs within eval()", {
-  skip_unless_utf8()
-
   e <- NULL
   trace <- NULL
 
@@ -400,26 +320,22 @@ test_that("collapsing of eval() frames detects when error occurs within eval()",
     error = calling(function(err) trace <<- trace_back(e))
   ))
 
-  expect_known_trace_output(trace, "test-trace-non-collapsed-eval")
+  expect_snapshot_trace(trace)
 })
 
 test_that("can print degenerate backtraces", {
-  skip_unless_utf8()
+  trace_sym <- new_trace(list(quote(foo)), int(0))
+  expect_snapshot_trace(trace_sym)
 
-  trace_sym <- new_trace(list(quote(foo)), int(0), chr(""))
-  expect_known_trace_output(trace_sym, file = "test-trace-degenerate-sym.txt")
+  trace_null <- new_trace(list(NULL), int(0))
+  expect_snapshot_trace(trace_null)
 
-  trace_null <- new_trace(list(NULL), int(0), chr(""))
-  expect_known_trace_output(trace_null, file = "test-trace-degenerate-null.txt")
-
-  trace_scalar <- new_trace(list(1L), int(0), chr(""))
-  expect_known_trace_output(trace_scalar, file = "test-trace-degenerate-scalar.txt")
+  trace_scalar <- new_trace(list(1L), int(0))
+  expect_snapshot_trace(trace_scalar)
 })
 
 test_that("check for dangling promise in call CAR (#492)", {
-  skip_unless_utf8()
-
-  expect_known_trace_output(file = "test-trace-call-car-promise.txt", local({
+  expect_snapshot_trace(local({
     e <- current_env()
 
     print.foo <- function(x) {
@@ -432,8 +348,6 @@ test_that("check for dangling promise in call CAR (#492)", {
 })
 
 test_that("dangling srcrefs are not printed", {
-  skip_unless_utf8()
-
   from <- test_path("fixtures", "trace-srcref.R")
   to <- test_path("fixtures", "trace-srcref2.R")
 
@@ -443,26 +357,24 @@ test_that("dangling srcrefs are not printed", {
   source(to, local = TRUE, keep.source = TRUE)
   unlink(to)
 
-  expect_known_trace_output(
+  expect_snapshot_trace(
     local(f(current_env())),
-    file ="test-trace-dangling-srcref.txt",
     srcrefs = TRUE
   )
 })
 
 test_that("summary.rlang_trace() prints the full tree", {
-  skip_unless_utf8()
-
   e <- current_env()
   f <- function() g()
   g <- function() h()
   h <- function() trace_back(e)
   trace <- f()
-  expect_known_output(summary(trace, srcrefs = FALSE), file = test_path("test-trace-summary.txt"))
+  expect_snapshot(summary(trace, srcrefs = FALSE))
 })
 
 test_that("unexported functions have `:::` prefix", {
-  skip_unless_utf8()
+  expect_true(TRUE)
+  return("no longer using the rlanglibtest")
 
   # Should be installed as part of the C API tests
   skip_if_not_installed("rlanglibtest")
@@ -472,32 +384,26 @@ test_that("unexported functions have `:::` prefix", {
   f <- function() test_trace_unexported_child(e)
   trace <- f()
 
-  expect_known_trace_output(trace, file = "test-trace-unexported-prefix.txt")
+  expect_snapshot_trace(trace)
 })
 
 test_that("global functions have `global::` prefix", {
-  skip_unless_utf8()
-
   f <- eval_bare(expr(function(e) rlang::trace_back(e)), global_env())
   g <- function(e) f(e)
   trace <- g(current_env())
 
-  expect_known_trace_output(trace, file = "test-trace-global-prefix.txt")
+  expect_snapshot_trace(trace)
 })
 
 test_that("local functions inheriting from global do not have `global::` prefix", {
-  skip_unless_utf8()
-
   f <- eval_bare(expr(function(e) rlang::trace_back(e)), env(global_env()))
   g <- function(e) f(e)
   trace <- g(current_env())
 
-  expect_known_trace_output(trace, file = "test-trace-local-prefix.txt")
+  expect_snapshot_trace(trace)
 })
 
 test_that("can trim layers of backtraces", {
-  skip_unless_utf8()
-
   e <- current_env()
   f <- function(n) identity(identity(g(n)))
   g <- function(n) identity(identity(h(n)))
@@ -508,7 +414,7 @@ test_that("can trim layers of backtraces", {
   trace2 <- f(2)
   trace3 <- f(3)
 
-  expect_known_output(file = test_path("test-trace-trim.txt"), {
+  expect_snapshot({
     local_options(rlang_trace_format_srcrefs = FALSE)
 
     cat_line("No trimming:")
@@ -539,90 +445,28 @@ test_that("can trim layers of backtraces", {
   expect_equal_trace(trace3, trace3_env)
 })
 
-test_that("can subset from bottom tree level", {
-  idx <- int(0, 1, 1, 1)
-  expect_identical(chain_indices(4L, idx), c(1L, 4L))
-  expect_identical(parents_indices(4L, idx), c(1L, 4L))
-  expect_identical(children_indices(4L, idx), int())
-
-  e <- current_env()
-  f <- function() identity(identity(g()))
-  g <- function() trace_back(e)
-  trace <- f()
-  out <- trace_subset_across(trace, 3)
-
-  f <- function() g()
-  expected <- f()
-
-  expect_equal_trace(out, expected)
-})
-
-test_that("can subset from top tree level", {
-  idx <- int(0, 1, 0, 3, 3, 3)
-  expect_identical(chain_indices(3L, idx), 3:6)
-  expect_identical(parents_indices(3L, idx), 3L)
-  expect_identical(children_indices(3L, idx), 4:6)
-
-  e <- current_env()
-  f <- function() identity(identity(g()))
-  g <- function() trace_back(e)
-  trace <- tryCatch(f())
-
-  out <- trace_subset_across(trace, -1, 1)
-  expected <- f()
-
-  expect_equal_trace(out, expected)
-})
-
-test_that("can subset in middle level", {
-  idx <- int(0, 1, 1, 1, 4, 4, 4)
-  expect_identical(chain_indices(3L, idx), c(1L, 3L))
-  expect_identical(parents_indices(3L, idx), c(1L, 3L))
-  expect_identical(children_indices(3L, idx), int())
-
-  e <- current_env()
-  f <- function() identity(identity(g()))
-  g <- function() identity(identity(h()))
-  h <- function() trace_back(e)
-  trace <- f()
-
-  out <- trace_subset_across(trace, 2, 2)
-  expect_equal(out$calls, alist(rlang:::f(), base::identity(g())))
-  expect_identical(out$parents, 0:1)
-
-
-  idx <- int(0, 1, 1, 1, 4, 4, 4)
-  expect_identical(chain_indices(4L, idx), c(1L, 4:7))
-  expect_identical(parents_indices(4L, idx), c(1L, 4L))
-  expect_identical(children_indices(4L, idx), 5:7)
-
-  out <- trace_subset_across(trace, 3, 2)
-  exp <- alist(
-    rlang:::f(),
-    rlang:::g(),
-    base::identity(identity(h())),
-    base::identity(h()),
-    rlang:::h()
-  )
-  expect_equal(out$calls, exp)
-  expect_identical(out$parents, c(0L, 1L, 2L, 2L, 2L))
-})
-
 test_that("fails when `bottom` is not on the stack", {
   expect_error(trace_back(bottom = env()), "Can't find `bottom`")
 })
 
 test_that("caught error does not display backtrace in knitted files", {
   skip_if_not_installed("knitr")
+  skip_if_not_installed("rmarkdown")
   skip_if(!rmarkdown::pandoc_available())
 
   local_options(
     rlang_backtrace_on_error = NULL,
+    rlang_backtrace_on_error_report = NULL,
     rlang_interactive = FALSE
   )
+
   lines <- render_md("test-trace.Rmd")
   error_line <- lines[[length(lines)]]
   expect_match(error_line, "foo$")
+  
+  expect_snapshot({
+    cat_line(render_md("test-trace-full.Rmd"))
+  })
 })
 
 test_that("empty backtraces are dealt with", {
@@ -656,5 +500,283 @@ test_that("can trace back with quosured symbol", {
 
   # FIXME: Weird trace structure
   trace <- f()
-  expect_is(trace, "rlang_trace")
+  expect_s3_class(trace, "rlang_trace")
+})
+
+test_that("can slice backtrace", {
+  trace <- new_trace(alist(a(), b(), c()), 0:2)
+
+  expect_identical(
+    trace_slice(trace, 2:3),
+    new_trace(alist(b(), c()), 0:1)
+  )
+
+  exp <- new_trace(alist(a(), c()), c(0L, 0L))
+  
+  expect_identical(
+    trace_slice(trace, c(1, 3)),
+    exp
+  )
+  expect_identical(
+    trace_slice(trace, -2),
+    exp
+  )
+})
+
+test_that("backtraces carry `version` attribute", {
+  expect_identical(attr(trace_back(), "version"), 2L)
+})
+
+test_that("can bind backtraces", {
+  trace1 <- new_trace(alist(a(), b(), c()), 0:2)
+
+  expect_equal(trace_bind(), new_trace(list(), int()))
+  expect_equal(trace_bind(trace1), trace1)
+  
+  trace2 <- new_trace(alist(foo(), bar(), baz()), c(0L, 1L, 1L))
+  out <- trace_bind(trace1, trace2)
+
+  expect_equal(
+    out$call,
+    alist(a(), b(), c(), foo(), bar(), baz())
+  )
+
+  expect_equal(
+    out$parent,
+    c(0:3, c(4L, 4L))
+  )
+})
+
+test_that("backtraces don't contain inlined objects (#1069, r-lib/testthat#1223)", {
+  # !! deparsing in older R
+  skip_if_not_installed("base", "3.5.0")
+
+  local_options(
+    rlang_trace_format_srcrefs = FALSE
+  )
+
+  e <- environment()
+  f <- function(...) do.call("g", list(runif(1e6) + 0))
+  g <- function(...) h()
+  h <- function() trace_back(e)
+  trace <- inject(f(!!list()))
+
+  expect_snapshot(summary(trace))
+  expect_lt(object.size(trace$call), 50000)
+})
+
+test_that("runs of namespaces are embolden (#946)", {
+  local_options(
+    rlang_trace_format_srcrefs = FALSE,
+    rlang_trace_top_env = current_env()
+  )
+  f <- function() g()
+  g <- function() h()
+  h <- function() identity(1 + "")
+  err <- catch_cnd(withCallingHandlers(f(), error = entrace), "error")
+
+  testthat::local_reproducible_output(crayon = TRUE)
+
+  expect_snapshot({
+    print(err)
+    summary(err)
+  })
+})
+
+test_that("`bottom` must be a positive integer", {
+  expect_snapshot((expect_error(trace_back(bottom = -1))))
+})
+
+test_that("collapsed case in branch formatting", {
+  trace <- new_trace(alist(f(), g(), h(), evalq(), evalq()), 0:4)
+  expect_snapshot_output(print(trace, simplify = "branch"))
+})
+
+test_that("can detect namespace and scope from call", {
+  fn <- set_env(function() NULL, empty_env())
+
+  expect_equal(
+    call_trace_context(quote(bar()), fn),
+    trace_context()
+  )
+  expect_equal(
+    call_trace_context(quote(foo::bar()), fn),
+    trace_context("foo", "::")
+  )
+  expect_equal(
+    call_trace_context(quote(foo:::bar()), fn),
+    trace_context("foo", ":::")
+  )
+})
+
+test_that("trailing `FALSE` visibility is handled", {
+  trace <- new_trace(
+    alist(f(), g(), h(), foo(), bar()),
+    parent = 0:4,
+    visible = c(TRUE, TRUE, TRUE, FALSE, FALSE)
+  )
+  expect_snapshot_trace(trace)
+})
+
+test_that("can create empty trace with trace_back()", {
+  expect_equal(
+    trace_back(top = environment()),
+    new_trace(list(), int())
+  )
+})
+
+test_that("can format empty traces", {
+  trace <- new_trace(list(), int())
+  expect_snapshot_trace(trace)
+})
+
+test_that("backtrace is formatted with sources (#1396)", {
+  file <- tempfile("my_source", fileext = ".R")
+  with_srcref(file = file, "
+    f <- function() g()
+    g <- function() abort('foo')
+  ")
+  err <- catch_cnd(f(), "error")
+
+  rlang_cli_local_hyperlinks()
+
+  lines <- format(err$trace)
+  n_links <- sum(grepl("\033]8;.*my_source.*\\.R:", lines))
+  expect_true(n_links > 0)
+})
+
+test_that("sibling streaks in tree backtraces", {
+  f <- function(x) identity(identity(x))
+  g <- function() f(f(h()))
+  h <- function() abort("foo")
+  err <- catch_cnd(f(g()), "error")
+  expect_snapshot_trace(err)
+})
+
+test_that("parallel '|' branches are correctly emphasised", {
+  f <- function(n) g(n)
+  g <- function(n) h(n)
+  h <- function(n) if (n) parallel(f(n - 1)) else abort("foo")
+  parallel <- function(x) p1(identity(x))
+  p1 <- function(x) p2(x)
+  p2 <- function(x) p3(x)
+  p3 <- function(x) x
+
+  err <- expect_error(parallel(f(0)))
+  expect_snapshot_trace(err)
+
+  deep <- function(n) parallel(f(n))
+  err <- expect_error(deep(1))
+  expect_snapshot_trace(err)
+})
+
+test_that("error calls and args are highlighted", {
+  f <- function(x) g(x)
+  g <- function(x) h(x)
+  h <- function(x) check_string(x)
+  wrapper <- function() {
+    try_fetch(f(1), error = function(cnd) abort("Tilt.", parent = cnd))
+  }
+
+  parent <- catch_error(f(1))
+  child <- catch_error(wrapper())
+
+  expect_snapshot({
+    print_highlighted_trace(parent)
+    print_highlighted_trace(child)
+  })
+})
+
+test_that("error calls and args are highlighted (no highlighted arg)", {
+  f <- function() g()
+  g <- function() h()
+  h <- function() abort("foo")
+
+  argless <- catch_error(f())
+
+  expect_snapshot({
+    print_highlighted_trace(argless)
+  })
+})
+
+test_that("frame is detected from the left", {
+  f <- function() g()
+  g <- function() h()
+  h <- function() identity(evalq(identity(abort("foo"))))
+  err <- catch_error(f())
+
+  expect_snapshot({
+    "If detected from the right, `evalq()`is highlighted instead of `h()`"
+    print_highlighted_trace(err)
+  })
+})
+
+test_that("arg is defensively checked", {
+  f <- function() g()
+  g <- function() h()
+  h <- function() abort("foo", arg = env())
+  err <- catch_error(f())
+
+  expect_snapshot({
+    print_highlighted_trace(err)
+  })
+})
+
+test_that("namespaced calls are highlighted", {
+  f <- function() g()
+  g <- function() h()
+  h <- function() rlang:::as_string(1)
+  err <- catch_error(f())
+
+  expect_snapshot({
+    print_highlighted_trace(err)
+  })
+})
+
+test_that("can highlight long lists of arguments in backtrace (#1456)", {
+  f <- function(...) g(
+    aaaaaaaaaaaa = aaaaaaaaaaaa,
+    bbbbbbbbbbbb = bbbbbbbbbbbb,
+    cccccccccccc = cccccccccccc,
+    dddddddddddd = dddddddddddd,
+    eeeeeeeeeeee = eeeeeeeeeeee,
+    ...
+  )
+  g <- function(aaaaaaaaaaaa,
+                bbbbbbbbbbbb,
+                cccccccccccc,
+                dddddddddddd,
+                eeeeeeeeeeee, ...) {
+    rlang::abort("foo", ...)
+  }
+
+  err <- catch_error(f())
+  expect_snapshot({
+    print_highlighted_trace(err)
+  })
+
+  err <- catch_error(f(arg = "bbbbbbbbbbbb"))
+  expect_snapshot({
+    print_highlighted_trace(err)
+  })
+})
+
+test_that("can highlight multi-line arguments in backtrace (#1456)", {
+  f <- function(...) g(x = {
+    a
+    b
+  }, ...)
+  g <- function(x, ...) {
+    rlang::abort("foo", ...)
+  }
+
+  err <- catch_error(f())
+  expect_snapshot({
+    print_highlighted_trace(err)
+  })
+
+  err <- catch_error(f(arg = "x"))
+  expect_snapshot({
+    print_highlighted_trace(err)
+  })
 })
